﻿B4J=true
Group=Controllers
ModulesStructureVersion=1
Type=Class
Version=10
@EndOfDesignText@
' MinimaList Controller
' Version 1.07
Sub Class_Globals
	Private Request As ServletRequest
	Private Response As ServletResponse
	Private HRM As HttpResponseMessage
	Private Method As String
	Private Version As String
	Private Elements() As String
	Private ApiVersionIndex As Int
	Private ControllerIndex As Int
	Private ElementLastIndex As Int
	Private FirstIndex As Int
	Private FirstElement As String
End Sub

Public Sub Initialize (req As ServletRequest, resp As ServletResponse)
	Request = req
	Response = resp
	HRM.Initialize
End Sub

Private Sub ReturnBadRequest
	WebApiUtils.ReturnBadRequest(Response)
End Sub

Private Sub ReturnApiResponse
	HRM.SimpleResponse = Main.SimpleResponse
	WebApiUtils.ReturnHttpResponse(HRM, Response)
End Sub

Private Sub ReturnMethodNotAllow
	WebApiUtils.ReturnMethodNotAllow(Response)
End Sub

Private Sub ReturnErrorUnprocessableEntity
	WebApiUtils.ReturnErrorUnprocessableEntity(Response)
End Sub

Public Sub Route (WebApi As String)
	Method = Request.Method.ToUpperCase
	Elements = WebApiUtils.GetUriElements(Request.RequestURI)
	ElementLastIndex = Elements.Length - 1
	
	Select WebApi.ToUpperCase
		Case "WEB"
			ControllerIndex = Main.Element.WebControllerIndex
		Case "API"
			ApiVersionIndex = Main.Element.ApiVersionIndex
			ControllerIndex = Main.Element.ApiControllerIndex
			Version = Elements(ApiVersionIndex)
			If ElementLastIndex > ControllerIndex Then
				FirstIndex = ControllerIndex + 1
				FirstElement = Elements(FirstIndex)
			End If
	End Select

	Select WebApi.ToUpperCase
		Case "API"
			Select Method
				Case "GET"
					RouteGet
				Case "POST"
					RoutePost
				Case "PUT"
					RoutePut
				Case "DELETE"
					RouteDelete
				Case Else
					Log("Unsupported method: " & Method)
					ReturnMethodNotAllow
			End Select
		Case Else
			ReturnBadRequest
	End Select
End Sub

' Snippet: Code_WebApiUtils_03 GET Route
Private Sub RouteGet
	Select Version
		Case "v2"
			Select ElementLastIndex
				Case ControllerIndex
					GetData
					Return
				Case FirstIndex
					If WebApiUtils.CheckInteger(FirstElement) = False Then
						ReturnErrorUnprocessableEntity
						Return
					End If
					GetOneData(FirstElement)
					Return
			End Select
	End Select
	ReturnBadRequest
End Sub

Private Sub RoutePost
	Select Version
		Case "v2"
			Select ElementLastIndex
				Case ControllerIndex
					PostData
					Return
			End Select
	End Select
	ReturnBadRequest
End Sub

Private Sub RoutePut
	Select Version
		Case "v2"
			Select ElementLastIndex
				Case FirstIndex
					If WebApiUtils.CheckInteger(FirstElement) = False Then
						ReturnErrorUnprocessableEntity
						Return
					End If
					PutData(FirstElement)
					Return
			End Select
	End Select
	ReturnBadRequest
End Sub

Private Sub RouteDelete
	Select Version
		Case "v2"
			Select ElementLastIndex
				Case FirstIndex
					If WebApiUtils.CheckInteger(FirstElement) = False Then
						ReturnErrorUnprocessableEntity
						Return
					End If
					DeleteData(FirstElement)
					Return
			End Select
	End Select
	ReturnBadRequest
End Sub

Private Sub GetData
	' #Version = v2
	' #Desc = Read all items in Data

	HRM.ResponseCode = 200
	HRM.ResponseData = Main.DataList.List
	ReturnApiResponse
End Sub

Private Sub GetOneData (id As Int)
	' #Version = v2
	' #Desc = Read one item in Data by id
	' #Elements = [":id"]

	Dim M1 As Map = Main.DataList.Find(id)
	If M1.Size > 0 Then
		HRM.ResponseCode = 200
		HRM.ResponseObject = M1
	Else
		HRM.ResponseCode = 404
		HRM.ResponseError = "Data not found"
	End If
	ReturnApiResponse
End Sub

Private Sub PostData
	' #Version = v2
	' #Desc = Add a new item into Data
	' #Body = {<br>&nbsp; "name": "value1",<br>&nbsp; "description": "value2"<br>}

	Dim data As Map = WebApiUtils.RequestData(Request)
	If Not(data.IsInitialized) Then
		HRM.ResponseCode = 400
		HRM.ResponseError = "Invalid json object"
		ReturnApiResponse
		Return
	End If
	
	If data.ContainsKey("") Then
		HRM.ResponseCode = 400
		HRM.ResponseError = "Invalid key value"
		ReturnApiResponse
		Return
	End If
	
	' Check conflict name
	Dim L1 As List = Main.DataList.FindAll(Array("name"), Array(data.Get("name")))
	If L1.Size > 0 Then
		HRM.ResponseCode = 409
		HRM.ResponseError = "Name already exist"
		ReturnApiResponse
		Return
	End If
		
	If Not(data.ContainsKey("created_date")) Then
		data.Put("created_date", WebApiUtils.CurrentDateTime)
	End If
		
	Main.DataList.Add(data)
	Main.WriteKVS("DataList", Main.DataList)
		
	HRM.ResponseCode = 201
	HRM.ResponseMessage = "Data Created"
	HRM.ResponseObject = Main.DataList.Last
	ReturnApiResponse
End Sub

Private Sub PutData (id As Int)
	' #Version = v2
	' #Desc = Update an item in DataList by Id
	' #Body = {<br>&nbsp; "name": "value1",<br>&nbsp; "description": "value2"<br>}
	' #Elements = [":id"]

	Dim data As Map = WebApiUtils.RequestData(Request)
	If Not(data.IsInitialized) Then
		HRM.ResponseCode = 400
		HRM.ResponseError = "Invalid json object"
		ReturnApiResponse
		Return
	End If
	
	If data.ContainsKey("") Then
		HRM.ResponseCode = 400
		HRM.ResponseError = "Invalid key value"
		ReturnApiResponse
		Return
	End If

	' Check conflict name
	Dim L1 As List = Main.DataList.FindAll(Array("name"), Array(data.Get("name")))
	For Each M As Map In L1
		If id <> M.Get("id") Then
			HRM.ResponseCode = 409
			HRM.ResponseError = "Name already exist"
			ReturnApiResponse
			Return
		End If
	Next

	Dim M1 As Map = Main.DataList.Find(id)
	If M1.Size > 0 Then
		If Not(data.ContainsKey("modified_date")) Then
			data.Put("modified_date", WebApiUtils.CurrentDateTime)
		End If
		
		For Each Key As String In data.Keys
			M1.Put(Key, data.Get(Key))
		Next
		Main.WriteKVS("DataList", Main.DataList)
				
		HRM.ResponseCode = 200
		HRM.ResponseMessage = "Data Updated"
		HRM.ResponseObject = M1
	Else
		HRM.ResponseCode = 404
		HRM.ResponseError = "Id not found"
	End If
	ReturnApiResponse
End Sub

Private Sub DeleteData (id As Int)
	' #Version = v2
	' #Desc = Remove an item in DataList by Id
	' #Elements = [":id"]

	Dim Index As Int = Main.DataList.IndexFromId(id)
	If Index > -1 Then
		Main.DataList.Remove(Index)
		Main.WriteKVS("DataList", Main.DataList)
		
		HRM.ResponseCode = 200
		HRM.ResponseMessage = "Data Deleted"
	Else
		HRM.ResponseCode = 404
		HRM.ResponseError = "Id not found"
	End If
	ReturnApiResponse
End Sub